import SSMotionControl
import tkinter as tk
import sys
from tkinter import StringVar
from tkinter import messagebox
from decimal import Decimal

import threading
import time
import datetime

SSMC_AXES_NUM = 8

drive_select_index = 0xFF
param_select_index = 0xFF
state_pressed = False

# FUNCTION

# UTIL
# 文字列をdecimalに変換
def is_decimal( str = '0' ):
    try:
        float(str)
    except ValueError:
        return Decimal('0')
    else:
        return Decimal(str)

# 文字列をintに変換
def is_int( str = '0' ):
    try:
        int(str)
    except ValueError:
        return int('-1')
    else:
        return int(str)

# 文字列をboolに変換
def is_bool( num = 0 ):
    if num == 0:
        return False
    else:
        return True

# 文字列をcur_on_offに変換
def is_cur_on_off( str = "" ):
    ON = "ON"
    OFF = "OFF"
    EMPTY = "empty"

    if ON.lower() == str.lower():
        return 0
    elif OFF.lower() == str.lower():
        return 1
    elif EMPTY.lower() == str.lower():
        return -1
    else:
        return -1

# 指定数、空白のある配列の入力値と位置(1～)を取得
def get_axis_decimal(count, axisNo, val):
    idx = 0
    try:
        for cnt, str in enumerate(dr_entry_val):
            if not str.get():
                continue
            else:
                axisNo[idx] = cnt + 1;
                val[idx] = is_decimal(str.get())
                idx += 1
                if idx >= count:
                    break
    except ValueError:
        print("frame error GetAxisDecimal");
    return idx

# 指定したウィジットに定型文ともに文字(リスト)を入力
def insert_list( handle=None, fixed="", Array=[] ):
    tmp=""
    for i, w in enumerate(Array):
        tmp+=fixed+"["+str(i)+"]: "+str(w)+'\r\n'

    if handle:
        handle.insert('end',tmp)
        handle.yview_moveto(1)

    return tmp

def insert_list_for_t_point( handle=None, fixed="", no=0, Array=[] ):
    tmp=""
    for i, w in enumerate(Array):
        if w == ssmc.POS_NC:
            tmp+=fixed+"(No."+str(no)+")"+"["+str(i)+"]: "+"N"+'\r\n'
        else:
            tmp+=fixed+"(No."+str(no)+")"+"["+str(i)+"]: "+str(w)+'\r\n'

    if handle:
        handle.insert('end',tmp)
        handle.yview_moveto(1)

    return tmp

# UI制御
# 駆動系リスト選択
def dr_listbox_selected(event):
    if len(dr_listbox.curselection()) == 0:
        return
    index = dr_listbox.curselection()[0]

    # 選択位置保存
    global drive_select_index
    drive_select_index = index

    if index == 0:      ## 原点復帰
        dr_input_pattern_change(1)   ## チェックボックスのみ

    elif index == 1:    ## ホームポジション
        dr_input_pattern_change(1)   ## チェックボックスのみ

    elif index == 2:    ## 絶対位置移動
        dr_input_pattern_change(2)   ## エントリーボックスのみ

    elif index == 3:    ## 相対移動(CW)
        dr_input_pattern_change(1)   ## チェックボックスのみ

    elif index == 4:    ## 相対移動(CCW)
        dr_input_pattern_change(1)   ## チェックボックスのみ

    elif index == 5:    ## 2軸直線補間移動
        dr_input_pattern_change(2)   ## エントリーボックスのみ

    elif index == 6:    ## 3軸直線補間移動
        dr_input_pattern_change(2)   ## エントリーボックスのみ

    elif index == 7:    ## 2軸円弧補間移動(CW)
        dr_input_pattern_change(2)   ## エントリーボックスのみ

    elif index == 8:    ## 2軸円弧補間移動(CCW)
        dr_input_pattern_change(2)   ## エントリーボックスのみ

    elif index == 9:    ## ティーチング位置移動
         dr_input_pattern_change(3)   ## スピンボックスのみ

    elif index == 10:   ## 速度設定
        dr_input_pattern_change(6)   ## エントリーボックス + オプション

    elif index == 11:   ## 目標位置設定
        dr_input_pattern_change(2)   ## エントリーボックスのみ

    elif index == 12:   ## 相対移動量設定
        dr_input_pattern_change(2)   ## エントリーボックスのみ

    elif index == 13:   ## 停止
        dr_input_pattern_change(1)   ## チェックボックスのみ

    elif index == 14:   ## 全軸停止
        dr_input_pattern_change(0)   ## すべて非活性

    elif index == 15:   ## 出力ポート制御
        dr_input_pattern_change(4)   ## ラジオボタンのみ

    elif index == 16:   ## 全ポート一括操作
        dr_input_pattern_change(5)   ## エントリーボックス2ボタンのみ

    elif index == 17:   ## 電流断設定
        dr_input_pattern_change(7)   ## エントリーボックス + オプション2

    else:
        print("エラー")

def dr_input_pattern_change(pattern = 0):
    if pattern == 0:   ## すべて非活性
        dr_entry_state_change('disable')
        dr_etc_spinbox_state_change('disable')
        dr_radio_btn_state_change('disable')
        dr_etc_entry_state_change('disable')
        dr_etc_entry_frame_str_visible(pattern)

    elif pattern == 1:   ## チェックボックスのみ
        dr_checkbutton_state_change('normal')
        dr_entry_state_change('disable')
        dr_etc_spinbox_state_change('disable')
        dr_radio_btn_state_change('disable')
        dr_etc_entry_state_change('disable')
        dr_etc_entry_frame_str_visible(pattern)

    elif pattern == 2:   ## エントリーボックスのみ
        dr_entry_state_change('normal')
        dr_etc_spinbox_state_change('disable')
        dr_radio_btn_state_change('disable')
        dr_etc_entry_state_change('disable')
        dr_etc_entry_frame_str_visible(pattern)

    elif pattern == 3:   ## スピンボックスのみ
        dr_entry_state_change('disable')
        dr_etc_spinbox_state_change('normal')
        dr_radio_btn_state_change('disable')
        dr_etc_entry_state_change('disable')
        dr_etc_entry_frame_str_visible(pattern)

    elif pattern == 4:   ## スピンボックス + ラジオボタンのみ
        dr_entry_state_change('disable')
        dr_etc_spinbox_state_change('normal')
        dr_radio_btn_state_change('normal')
        dr_etc_entry_state_change('disable')
        dr_etc_entry_frame_str_visible(pattern)

    elif pattern == 5:   ## エントリーボックス2ボタンのみ
        dr_entry_state_change('disable')
        dr_etc_spinbox_state_change('disable')
        dr_radio_btn_state_change('disable')
        dr_etc_entry_state_change('normal')
        dr_etc_entry_frame_str_visible(pattern)

    elif pattern == 6:   ## エントリーボックス + オプション
        dr_entry_state_change('normal')
        dr_etc_spinbox_state_change('disable')
        dr_radio_btn_state_change('disable')
        dr_etc_entry_state_change('disable')
        dr_etc_entry_frame_str_visible(pattern)

    elif pattern == 7:   ## エントリーボックス + オプション2
        dr_entry_state_change('normal')
        dr_etc_spinbox_state_change('disable')
        dr_radio_btn_state_change('disable')
        dr_etc_entry_state_change('disable')
        dr_etc_entry_frame_str_visible(pattern)

def dr_checkbutton_state_change(type = 'normal'):
    dr_checkbutton1.config(state=type)
    dr_checkbutton2.config(state=type)
    dr_checkbutton3.config(state=type)
    dr_checkbutton4.config(state=type)
    dr_checkbutton5.config(state=type)
    dr_checkbutton6.config(state=type)
    dr_checkbutton7.config(state=type)
    dr_checkbutton8.config(state=type)
    
def dr_entry_state_change(type = 'normal'):
    dr_entry1.config(state=type)
    dr_entry2.config(state=type)
    dr_entry3.config(state=type)
    dr_entry4.config(state=type)
    dr_entry5.config(state=type)
    dr_entry6.config(state=type)
    dr_entry7.config(state=type)
    dr_entry8.config(state=type)

def dr_etc_spinbox_state_change(type = 'normal'):
    dr_etc_spinbox.config(state=type)

def dr_radio_btn_state_change(type = 'normal'):
    dr_radio_btn1.config(state=type)
    dr_radio_btn2.config(state=type)

def dr_etc_entry_state_change(type = 'normal'):
    dr_etc_entry1.config(state=type)

def dr_etc_entry_frame_str_visible(pattern = 0):
    if pattern == 6:   # 速度設定
        dr_etc_label_str.set("0～15: テーブル番号\r\n-1: 設定しない")
    elif pattern == 7:   # 電流断設定
        dr_etc_label_str.set("ON: CUROFF\r\nOFF: CURON\r\nempty: NotControl")
    else:
        dr_etc_label_str.set("")

# 実行制御
# 駆動系実行ボタン
def dr_button_exec():
    index = drive_select_index

    if index == 0:      # 原点復帰
        err = ssmc.GO_ORG(
        dr_check_val[0].get(),
        dr_check_val[1].get(),
        dr_check_val[2].get(),
        dr_check_val[3].get(),
        dr_check_val[4].get(),
        dr_check_val[5].get(),
        dr_check_val[6].get(),
        dr_check_val[7].get(),
        )
        if not err:
            print("Failed to GO_ORG()")

    elif index == 1:    # ホームポジション
        err = ssmc.GO_HOME(
        dr_check_val[0].get(),
        dr_check_val[1].get(),
        dr_check_val[2].get(),
        dr_check_val[3].get(),
        dr_check_val[4].get(),
        dr_check_val[5].get(),
        dr_check_val[6].get(),
        dr_check_val[7].get(),
        )
        if not err:
            print("Failed to GO_HOME()")

    elif index == 2:    # 絶対位置移動
        err = ssmc.GO_ABS(
        is_decimal(dr_entry_val[0].get()),
        is_decimal(dr_entry_val[1].get()),
        is_decimal(dr_entry_val[2].get()),
        is_decimal(dr_entry_val[3].get()),
        is_decimal(dr_entry_val[4].get()),
        is_decimal(dr_entry_val[5].get()),
        is_decimal(dr_entry_val[6].get()),
        is_decimal(dr_entry_val[7].get()),
        )
        if not err:
            print("Failed to GO_ABS()")

    elif index == 3:    # 相対移動(CW)
        err = ssmc.GO_CW(
        dr_check_val[0].get(),
        dr_check_val[1].get(),
        dr_check_val[2].get(),
        dr_check_val[3].get(),
        dr_check_val[4].get(),
        dr_check_val[5].get(),
        dr_check_val[6].get(),
        dr_check_val[7].get(),
        )
        if not err:
            print("Failed to GO_CW()")

    elif index == 4:    # 相対移動(CCW)
        err = ssmc.GO_CCW(
        dr_check_val[0].get(),
        dr_check_val[1].get(),
        dr_check_val[2].get(),
        dr_check_val[3].get(),
        dr_check_val[4].get(),
        dr_check_val[5].get(),
        dr_check_val[6].get(),
        dr_check_val[7].get(),
        )
        if not err:
            print("Failed to GO_CCW()")

    elif index == 5:    # 2軸直線補間移動
        axis_no = [0] * 2
        position_val = [0.0] * 2

        if not get_axis_decimal(2, axis_no, position_val) == 0: 
            err = ssmc.GO_LIN2(
                axis_no[0],
                position_val[0],
                axis_no[1],
                position_val[1],
                )
            if not err:
                print("Failed to GO_LIN2()")

    elif index == 6:    # 3軸直線補間移動
        axis_no = [0] * 3
        position_val = [0.0] * 3

        if not get_axis_decimal(3, axis_no, position_val) == 0: 
            err = ssmc.GO_LIN3(
                axis_no[0],
                position_val[0],
                axis_no[1],
                position_val[1],
                axis_no[2],
                position_val[2],
                )
            if not err:
                print("Failed to GO_LIN3()")

    elif index == 7:    # 2軸円弧補間移動(CW)
        axis_no = [0] * 2
        position_val = [0.0] * 2

        if not get_axis_decimal(2, axis_no, position_val) == 0: 
            err = ssmc.GO_CIRCW(
                axis_no[0],
                position_val[0],
                axis_no[1],
                position_val[1],
                )
            if not err:
                print("Failed to GO_CIRCW()")

    elif index == 8:    # 2軸円弧補間移動(CCW)
        axis_no = [0] * 2
        position_val = [0.0] * 2

        if not get_axis_decimal(2, axis_no, position_val) == 0: 
            err = ssmc.GO_CIRCCW(
                axis_no[0],
                position_val[0],
                axis_no[1],
                position_val[1],
                )
            if not err:
                print("Failed to GO_CIRCCW()")

    elif index == 9:    # ティーチング位置移動
        err = ssmc.GO_TCH(is_decimal(dr_etc_spin_var.get()))
        if not err:
            print("Failed to GO_TCH()")

    elif index == 10:   # 速度設定
        num = is_int(dr_entry_val[0].get())
        print(num)

        err = ssmc.SEL_SPEED(
        is_int(dr_entry_val[0].get()),
        is_int(dr_entry_val[1].get()),
        is_int(dr_entry_val[2].get()),
        is_int(dr_entry_val[3].get()),
        is_int(dr_entry_val[4].get()),
        is_int(dr_entry_val[5].get()),
        is_int(dr_entry_val[6].get()),
        is_int(dr_entry_val[7].get()),
        )
        if not err:
            print("Failed to SEL_SPEED()")

    elif index == 11:   # 目標位置設定
        err = ssmc.SET_TGPOS(
        is_decimal(dr_entry_val[0].get()),
        is_decimal(dr_entry_val[1].get()),
        is_decimal(dr_entry_val[2].get()),
        is_decimal(dr_entry_val[3].get()),
        is_decimal(dr_entry_val[4].get()),
        is_decimal(dr_entry_val[5].get()),
        is_decimal(dr_entry_val[6].get()),
        is_decimal(dr_entry_val[7].get()),
        )
        if not err:
            print("Failed to SET_TGPOS()")

    elif index == 12:   # 相対移動量設定
        err = ssmc.SET_RELLENGTH(
        is_decimal(dr_entry_val[0].get()),
        is_decimal(dr_entry_val[1].get()),
        is_decimal(dr_entry_val[2].get()),
        is_decimal(dr_entry_val[3].get()),
        is_decimal(dr_entry_val[4].get()),
        is_decimal(dr_entry_val[5].get()),
        is_decimal(dr_entry_val[6].get()),
        is_decimal(dr_entry_val[7].get()),
        )
        if not err:
            print("Failed to SET_RELLENGTH()")

    elif index == 13:   # 停止
        err = ssmc.STOP(
        dr_check_val[0].get(),
        dr_check_val[1].get(),
        dr_check_val[2].get(),
        dr_check_val[3].get(),
        dr_check_val[4].get(),
        dr_check_val[5].get(),
        dr_check_val[6].get(),
        dr_check_val[7].get(),
        )
        if not err:
            print("Failed to STOP()")

    elif index == 14:   # 全軸停止
        err = ssmc.STOP_ALL()
        if not err:
            print("Failed to STOP_ALL()")

    elif index == 15:   # 出力ポート制御
        err = ssmc.OUTP(
        is_decimal(dr_etc_spin_var.get()),
        is_bool(dr_radio_var.get())
        )
        if not err:
            print("Failed to OUTP()")

    elif index == 16:   # 全ポート一括操作
        ret = int(dr_etc_entry1.get(), 16)
        print(ret)

        err = ssmc.OUTP_ALL(int(dr_etc_entry1.get(), 16))
        if not err:
            print("Failed to OUTP_ALL()")

    elif index == 17:   # 電流断設定
        err = ssmc.SET_CURON(
        is_cur_on_off(dr_entry_val[0].get()),
        is_cur_on_off(dr_entry_val[1].get()),
        is_cur_on_off(dr_entry_val[2].get()),
        is_cur_on_off(dr_entry_val[3].get()),
        is_cur_on_off(dr_entry_val[4].get()),
        is_cur_on_off(dr_entry_val[5].get()),
        is_cur_on_off(dr_entry_val[6].get()),
        is_cur_on_off(dr_entry_val[7].get()),
        )
        if not err:
            print("Failed to SET_CURON()")

    else:
        print("Failed to dr_button_exec()")

# 状態制御

# 状態取得処理
def st_get_state():
    global state_pressed
    
    if state_pressed == True:
        err, busy, ml_ccw, sl_ccw, sl_cw, mi_cw, comp_org, emergency = ssmc.GetStatus()
        if not err:
            print("Failed to GetStatus()")
            return
        
        err, pos = ssmc.GetPosition()
        if not err:
            print("Failed to GetPosition()")
            return
        
        err, state = ssmc.GetInPort(0)
        if not err:
            print("Failed to GetInPort()")
            return
        
        err, word = ssmc.GetInPortAll()
        if not err:
            print("Failed to GetInPortAll()")
            return

        #各パラメータ設定
        for i in range(len(busy)):
            if (dr_check_val[0].get() and i == 0) or \
               (dr_check_val[1].get() and i == 1) or \
               (dr_check_val[2].get() and i == 2) or \
               (dr_check_val[3].get() and i == 3) or \
               (dr_check_val[4].get() and i == 4) or \
               (dr_check_val[5].get() and i == 5) or \
               (dr_check_val[6].get() and i == 6) or \
               (dr_check_val[7].get() and i == 7):

                st_busy_var[i].set(busy[i])
                st_ml_ccw_var[i].set(ml_ccw[i])
                st_sl_ccw_var[i].set(sl_ccw[i])
                st_ml_cw_var[i].set(mi_cw[i])
                st_sl_cw_var[i].set(sl_cw[i])
                st_comp_org_var[i].set(comp_org[i])
                st_entry_val[i].set(str(pos[i]))

            if i < 4:
                unit_state_id0_cb_var[i].set(True)
            else:
                unit_state_id1_cb_var[i-4].set(True)

        for i in range(len(emergency)):
            st_em_id_var[i].set(emergency[i])

        st_port_id0_cb_var[0].set((word & 1) != 0)
        st_port_id0_cb_var[1].set((word & 2) != 0)
        st_port_id0_cb_var[2].set((word & 4) != 0)
        st_port_id0_cb_var[3].set((word & 8) != 0)
        st_port_id0_cb_var[4].set((word & 0x10) != 0)
        st_port_id0_cb_var[5].set((word & 0x20) != 0)
        st_port_id0_cb_var[6].set((word & 0x40) != 0)
        st_port_id0_cb_var[7].set((word & 0x80) != 0)
        st_port_id1_cb_var[0].set((word & 0x100) != 0)
        st_port_id1_cb_var[1].set((word & 0x200) != 0)
        st_port_id1_cb_var[2].set((word & 0x400) != 0)
        st_port_id1_cb_var[3].set((word & 0x800) != 0)
        st_port_id1_cb_var[4].set((word & 0x1000) != 0)
        st_port_id1_cb_var[5].set((word & 0x2000) != 0)
        st_port_id1_cb_var[6].set((word & 0x4000) != 0)
        st_port_id1_cb_var[7].set((word & 0x8000) != 0)

        # 移動中の軸が1つでもあったら、移動待ちを継続
        if  True in busy:
            return

        state_pressed = False
        dr_button.config(state='normal')
        st_button_str.set("状態取得と移動待ち")

# 状態系実行ボタン
def st_button_exec():
    global state_pressed

    if state_pressed == False:
        state_pressed = True
        dr_button.config(state='disable')
        st_button_str.set("移動待ちキャンセル")

    else:
        state_pressed = False
        dr_button.config(state='normal')
        st_button_str.set("状態取得と移動待ち")

class Worker(threading.Thread):
    def __init__(self, fn):
        threading.Thread.__init__(self)
        self.event = threading.Event()
        self.fn = fn

    def run(self):
        while not self.event.is_set():
            self.fn()
            self.event.wait(0.5)

    def stop(self):
        self.event.set()

# UI制御
# 設定/取得系リスト選択
def pr_listbox_selected(event):
    if len(pr_listbox.curselection()) == 0:
        return
    index = pr_listbox.curselection()[0]

    # 選択位置保存
    global param_select_index
    param_select_index = index

# コントローラID設定
def unit_button_exec():
    id = [0] * len(unit_id_select_label_str)

    for i in range(len(unit_id_select_label_str)):
        if unit_id_select_num[i].get() == unit_id_select_label_str[0]:
            id[i] = 0
        elif  unit_id_select_num[i].get() == unit_id_select_label_str[1]:
            id[i] = 1
    else:
        print("Failed to unit_button_exec()")

    link = unit_link_val.get()

    err = ssmc.SetID(id, link)
    if not err:
        print("Failed to SetID()")
        return

    global state_pressed
    if state_pressed == False:
        state_pressed = True

# コントーラID更新
def unit_update_exec():
    ssmc.Close()

    time.sleep(1)

    ssmc.Open()

    err, id, link = ssmc.GetID()

    if not err:
        print("Failed to GetID()")
        return

    for i in range(len(unit_id_select_label_str)):
        if id[i] == True:
            unit_id_select_num[i].set(unit_id_select_str[i])
        else:
            unit_id_select_num[i].set('')

    unit_link_val.set(link)

    for i in range(len(unit_state_id_str)):
        unit_state_id0_cb_var[i].set(False)
        unit_state_id1_cb_var[i].set(False)

    global state_pressed
    if state_pressed == False:
        state_pressed = True

# パラメータ取得／設定
def pr_button_exec():
    index = param_select_index

    if index == 0:      # ステージパラメータ
        err, originReturnPattern, isLimitSensorOpen, isOriginSensorOpen, isProximityOriginSensorOpen, isEncoderEntryEnable, isDriveDirectionReverse, stageType, motionRadius = ssmc.GetStageParameter()
        if not err:
            print("Failed to GetStageParameter()")
            return False
        insert_list(handle=logger_text, fixed="noriginReturnPattern",Array=originReturnPattern)
        insert_list(handle=logger_text, fixed="isLimitSensorOpen",Array=isLimitSensorOpen)
        insert_list(handle=logger_text, fixed="isOriginSensorOpen",Array=isOriginSensorOpen)
        insert_list(handle=logger_text, fixed="isProximityOriginSensorOpen",Array=isProximityOriginSensorOpen)
        insert_list(handle=logger_text, fixed="isEncoderEntryEnable",Array=isEncoderEntryEnable)
        insert_list(handle=logger_text, fixed="isDriveDirectionReverse",Array=isDriveDirectionReverse)
        insert_list(handle=logger_text, fixed="stageType",Array=stageType)
        insert_list(handle=logger_text, fixed="motionRadius",Array=motionRadius)
        err = ssmc.SetStageParameter(originReturnPattern
            , isLimitSensorOpen, isOriginSensorOpen, isProximityOriginSensorOpen
            , isEncoderEntryEnable, isDriveDirectionReverse, stageType, motionRadius)
        if not err:
            print("Failed to SetStageParameter()")

    elif index == 1:    # 単位
        err, unit = ssmc.GetUnit()
        if not err:
            print("Failed to GetUnit()")
            return False
        insert_list(handle=logger_text, fixed="Unit",Array=unit)
        err = ssmc.SetUnit(unit)
        if not err:
            print("Failed to SetUnit()")

    elif index == 2:    # フルステップ時の1パルス移動量
        err, value = ssmc.GetTravelPerPulseByFullstep()
        if not err:
            print("Failed to GetTravelPerPulseByFullstep()")
            return False
        insert_list(handle=logger_text, fixed="TravelPerPulseByFullstep",Array=value)
        err = ssmc.SetTravelPerPulseByFullstep(value)
        if not err:
            print("Failed to SetTravelPerPulseByFullstep()")

    elif index == 3:    # ドライバ分割数
        err, value, isAngleDiv = ssmc.GetStepAngleDirectly()
        if not err:
            print("Failed to GetStepAngleDirectly()")
            return False
        insert_list(handle=logger_text, fixed="StepAngleDirectly",Array=value)
        insert_list(handle=logger_text, fixed="isAngleDiv",Array=isAngleDiv)
        err = ssmc.SetStepAngleDirectly(value, isAngleDiv)
        if not err:
            print("Failed to SetStepAngleDirectly()")

    elif index == 4:    # 起動速度(原点復帰時)
        err, pps = ssmc.GetStartupSpeedForOriginReturn()
        if not err:
            print("Failed to GetStartupSpeedForOriginReturn()")
            return False
        insert_list(handle=logger_text, fixed="StartupSpeedForOriginReturn",Array=pps)
        err = ssmc.SetStartupSpeedForOriginReturn(pps)
        if not err:
            print("Failed to SetStartupSpeedForOriginReturn()")

    elif index == 5:    # 駆動速度(原点復帰時)
        err, pps = ssmc.GetDrivingSpeedForOriginReturn()
        if not err:
            print("Failed to GetDrivingSpeedForOriginReturn()")
            return False
        insert_list(handle=logger_text, fixed="DrivingSpeedForOriginReturn",Array=pps)
        err = ssmc.SetDrivingSpeedForOriginReturn(pps)
        if not err:
            print("Failed to SetDrivingSpeedForOriginReturn()")

    elif index == 6:    # 加速時間(原点復帰時)
        err, msec = ssmc.GetAccelerationTimeForOriginReturn()
        if not err:
            print("Failed to GetAccelerationTimeForOriginReturn()")
            return False
        insert_list(handle=logger_text, fixed="AccelerationTimeForOriginReturn",Array=msec)
        err = ssmc.SetAccelerationTimeForOriginReturn(msec)
        if not err:
            print("Failed to SetAccelerationTimeForOriginReturn()")

    elif index == 7:    # S字レート(原点復帰時)
        err, rate = ssmc.GetSCurveRateForOriginReturn()
        if not err:
            print("Failed to GetSCurveRateForOriginReturn()")
            return False
        insert_list(handle=logger_text, fixed="SCurveRateForOriginReturn",Array=rate)
        err = ssmc.SetSCurveRateForOriginReturn(rate)
        if not err:
            print("Failed to SetSCurveRateForOriginReturn()")

    elif index == 8:    # エンコーダ設定
        err, multiplier, isCountDirectionInvert, isZPhaseActiveLow, numberOfPositioningAdjustment = ssmc.GetEncoderParameter()
        if not err:
            print("Failed to GetEncoderParameter()")
            return False
        insert_list(handle=logger_text, fixed="multiplier",Array=multiplier)
        insert_list(handle=logger_text, fixed="isCountDirectionInvert",Array=isCountDirectionInvert)
        insert_list(handle=logger_text, fixed="isZPhaseActiveLow",Array=isZPhaseActiveLow)
        insert_list(handle=logger_text, fixed="numberOfPositioningAdjustment",Array=numberOfPositioningAdjustment)
        err = ssmc.SetEncoderParameter(multiplier
            , isCountDirectionInvert, isZPhaseActiveLow, numberOfPositioningAdjustment)
        if not err:
            print("Failed to SetEncoderParameter()")

    elif index == 9:    # エンコーダ分解能比率分子
        err, value = ssmc.GetEncoderResolutionNumerator()
        if not err:
            print("Failed to GetEncoderResolutionNumerator()")
            return False
        insert_list(handle=logger_text, fixed="EncoderResolutionNumerator",Array=value)
        err = ssmc.SetEncoderResolutionNumerator(value)
        if not err:
            print("Failed to SetEncoderResolutionNumerator()")

    elif index == 10:   # エンコーダ分解能比率分母
        err, value = ssmc.GetEncoderResolutionDenominator()
        if not err:
            print("Failed to GetEncoderResolutionDenominator()")
            return False
        insert_list(handle=logger_text, fixed="EncoderResolutionDenominator",Array=value)
        err = ssmc.SetEncoderResolutionDenominator(value)
        if not err:
            print("Failed to SetEncoderResolutionDenominator()")

    elif index == 11:   # ソフトリミット有効/無効
        err, enabled = ssmc.GetSoftwareLimit()
        if not err:
            print("Failed to GetSoftwareLimit()")
            return False
        insert_list(handle=logger_text, fixed="SoftwareLimit",Array=enabled)
        err = ssmc.SetSoftwareLimit(enabled)
        if not err:
            print("Failed to SetSoftwareLimit()")

    elif index == 12:   # CW側ソフトリミット
        err, value = ssmc.GetCWLS()
        if not err:
            print("Failed to GetCWLS()")
            return False
        insert_list(handle=logger_text, fixed="CWLS",Array=value)
        err = ssmc.SetCWLS(value)
        if not err:
            print("Failed to SetCWLS()")

    elif index == 13:   # CCW側ソフトリミット
        err, value = ssmc.GetCCWLS()
        if not err:
            print("Failed to GetCCWLS()")
            return False
        insert_list(handle=logger_text, fixed="CCWLS",Array=value)
        err = ssmc.SetCCWLS(value)
        if not err:
            print("Failed to SetCCWLS()")

    elif index == 14:   # ホームポジション
        err, value = ssmc.GetHomePosition()
        if not err:
            print("Failed to GetHomePosition()")
            return False
        insert_list(handle=logger_text, fixed="HomePosition",Array=value)
        err = ssmc.SetHomePosition(value)
        if not err:
            print("Failed to SetHomePosition()")

    elif index == 15:   # ステージ機種ID
        err, id = ssmc.GetStageID()
        if not err:
            print("Failed to GetStageID()")
            return False
        insert_list(handle=logger_text, fixed="StageID",Array=id)
        err = ssmc.SetStageID(id)
        if not err:
            print("Failed to SetStageID()")

    elif index == 16:   # 起動速度(速度テーブル)
        err, pps = ssmc.GetStartupSpeed()
        if not err:
            print("Failed to GetStartupSpeed()")
            return False
        insert_list(handle=logger_text, fixed="StartupSpeed",Array=pps)
        err = ssmc.SetStartupSpeed(pps)
        if not err:
            print("Failed to SetStartupSpeed()")

    elif index == 17:   # 駆動速度(速度テーブル)
        err, pps = ssmc.GetDrivingSpeed()
        if not err:
            print("Failed to GetDrivingSpeed()")
            return False
        insert_list(handle=logger_text, fixed="DrivingSpeed",Array=pps)
        err = ssmc.SetDrivingSpeed(pps)
        if not err:
            print("Failed to SetDrivingSpeed()")

    elif index == 18:   # 加速時間(速度テーブル) 
        err, msec = ssmc.GetAccelerationTime()
        if not err:
            print("Failed to GetAccelerationTime()")
            return False
        insert_list(handle=logger_text, fixed="AccelerationTime",Array=msec)
        err = ssmc.SetAccelerationTime(msec)
        if not err:
            print("Failed to SetAccelerationTime()")

    elif index == 19:   # S字レート(速度テーブル) 
        err, rate = ssmc.GetSCurveRate()
        if not err:
            print("Failed to GetSCurveRate()")
            return False
        insert_list(handle=logger_text, fixed="SCurveRate",Array=rate)
        err = ssmc.SetSCurveRate(rate)
        if not err:
            print("Failed to SetSCurveRate()")

    elif index == 20:   # ティーチングポイント 
        num = is_int(pr_tp_num_spin_var.get())
        err, position = ssmc.GetTeachingPoint(is_int(pr_tp_num_spin_var.get()))
        if not err:
            print("Failed to GetTeachingPoint()")
            return False
        insert_list_for_t_point(handle=logger_text, fixed="TeachingPoint", no=is_int(pr_tp_num_spin_var.get()), Array=position)
        err = ssmc.SetTeachingPoint(is_int(pr_tp_num_spin_var.get()), *position)
        if not err:
            print("Failed to SetTeachingPoint()")

    elif index == 21:   # フラッシュメモリ書き込み 
        if messagebox.askyesno("フラッシュメモリ書き込み", "フラッシュメモリ書き込みは数秒かかります\r\n続行しますか？"):

            err = ssmc.WriteFlashMemory()
            if not err:
                print("Failed to WriteFlashMemory()")
            else:
                messagebox.showinfo("フラッシュメモリ書き込み", "完了")

    else:
        print("Failed to pr_button_exec()")

# FRAME
# メインウィンドウ
root = tk.Tk()
root.title(u"Form1")
root.geometry("+{}+{}".format(50, 50))
root.grid()

# ウインドウクローズ
def on_closing():
    worker_thread.stop()
    ssmc.Close()
    sys.exit()

root.protocol("WM_DELETE_WINDOW", on_closing)

# サブウィンドウ
drive_state = tk.Frame(root, width=300, height=300 )
drive_state.grid(column=0, row=0, sticky='n' )

param_logger = tk.Frame(root, width=300, height=300 )
param_logger.grid(column=1, row=0, sticky='n' )

drive = tk.LabelFrame(drive_state, width=300, height=300,text="駆動")
drive.grid(column=0, row=0, sticky='news' )

state = tk.LabelFrame(drive_state, width=300, height=300,text="状態")
state.grid(column=0, row=1, sticky='news' )

unit = tk.LabelFrame(drive_state, width=300, height=300,text="コントローラID設定")
unit.grid(column=0, row=2, sticky='news' )

param = tk.LabelFrame(param_logger, width=300, height=300,text="設定/取得")
param.grid(column=0, row=0, sticky='news' )

logger = tk.Frame(param_logger, width=300, height=300 )
logger.grid(column=0, row=1, sticky='news' )

# ドライブ > サブウィンドウ
dr_sub = tk.Frame(drive, width=300, height=300 )
dr_sub.grid(column=0, row=1, sticky='news' )

# ドライブ > サブウィンドウ > 選択
dr_select = tk.Frame(drive, width=300, height=300 )
dr_select.grid(column=0, row=1, sticky='news' )

# ドライブ > サブウィンドウ > 選択 > 1.リスト
dr_ver = StringVar(value=( "原点復帰",
                        "ホームポジション",
                        "絶対位置移動",
                        "相対移動(CW)",
                        "相対移動(CCW)",
                        "2軸直線補間移動",
                        "3軸直線補間移動",
                        "2軸円弧補間移動(CW)",
                        "2軸円弧補間移動(CCW)",
                        "ティーチング位置移動",
                        "速度設定",
                        "目標位置設定",
                        "相対移動量設定",
                        "停止",
                        "全軸停止",
                        "出力ポート制御",
                        "全ポート一括操作",
                        "電流断設定" ) )

dr_listbox = tk.Listbox(dr_select,
                     listvariable=dr_ver,
                     height=18, command=None)
                     
dr_listbox.bind('<<ListboxSelect>>', dr_listbox_selected)
dr_listbox.grid(column=0, row=0, sticky='n' )

# ドライブ > サブウィンドウ > 選択 > 種別
dr_kind = tk.Frame(dr_select, width=300, height=300 )
dr_kind.grid(column=1, row=0, sticky='news' )

# ドライブ > サブウィンドウ > 選択 > 種別 > パラメータ
dr_parameter = tk.Frame(dr_kind, width=300, height=300 )
dr_parameter.grid(column=0, row=0, sticky='news' )

# ドライブ > サブウィンドウ > 選択 > 種別 > 状態
dr_state = tk.Frame(dr_kind, width=300, height=300 )
dr_state.grid(column=0, row=1, sticky='news' )

# ドライブ > 2.実行
dr_button = tk.Button(dr_parameter, text="実行", width=8, command=dr_button_exec)
dr_button.grid(column=0, row=0, sticky='s' )

# ドライブ > 3.軸選択
dr_ax_lab_frame = tk.LabelFrame(dr_parameter, width=300, height=300,text="軸選択")
dr_ax_lab_frame.grid(column=1, row=0, sticky=tk.N + tk.S)

# ドライブ > 3.軸選択 > チェックボックス
dr_check_val = [False] * SSMC_AXES_NUM

dr_check_val[0] = tk.BooleanVar()
dr_check_val[1] = tk.BooleanVar()
dr_check_val[2] = tk.BooleanVar()
dr_check_val[3] = tk.BooleanVar()
dr_check_val[4] = tk.BooleanVar()
dr_check_val[5] = tk.BooleanVar()
dr_check_val[6] = tk.BooleanVar()
dr_check_val[7] = tk.BooleanVar()

dr_check_val[0].set(False)
dr_check_val[1].set(False)
dr_check_val[2].set(False)
dr_check_val[3].set(False)
dr_check_val[4].set(False)
dr_check_val[5].set(False)
dr_check_val[6].set(False)
dr_check_val[7].set(False)

dr_checkbutton1 = tk.Checkbutton(dr_ax_lab_frame,text=u"軸1",variable=dr_check_val[0])
dr_checkbutton1.grid(column=0, row=0)

dr_checkbutton2 = tk.Checkbutton(dr_ax_lab_frame,text=u"軸2",variable=dr_check_val[1])
dr_checkbutton2.grid(column=0, row=1)

dr_checkbutton3 = tk.Checkbutton(dr_ax_lab_frame,text=u"軸3",variable=dr_check_val[2])
dr_checkbutton3.grid(column=0, row=2)

dr_checkbutton4 = tk.Checkbutton(dr_ax_lab_frame,text=u"軸4",variable=dr_check_val[3])
dr_checkbutton4.grid(column=0, row=3)

dr_checkbutton5 = tk.Checkbutton(dr_ax_lab_frame,text=u"軸5",variable=dr_check_val[4])
dr_checkbutton5.grid(column=0, row=4)

dr_checkbutton6 = tk.Checkbutton(dr_ax_lab_frame,text=u"軸6",variable=dr_check_val[5])
dr_checkbutton6.grid(column=0, row=5)

dr_checkbutton7 = tk.Checkbutton(dr_ax_lab_frame,text=u"軸7",variable=dr_check_val[6])
dr_checkbutton7.grid(column=0, row=6)

dr_checkbutton8 = tk.Checkbutton(dr_ax_lab_frame,text=u"軸8",variable=dr_check_val[7])
dr_checkbutton8.grid(column=0, row=7)


# ドライブ > 4.軸位置/移動量
dr_ax_lab_frame = tk.LabelFrame(dr_parameter, width=300, height=300,text="軸位置/移動量")
dr_ax_lab_frame.grid(column=2, row=0, sticky=tk.N + tk.S)

# ドライブ > 4.軸位置/移動量 > インプットボックス
dr_entry_val = [0] * SSMC_AXES_NUM
dr_entry_val[0] = tk.StringVar()
dr_entry_val[1] = tk.StringVar()
dr_entry_val[2] = tk.StringVar()
dr_entry_val[3] = tk.StringVar()
dr_entry_val[4] = tk.StringVar()
dr_entry_val[5] = tk.StringVar()
dr_entry_val[6] = tk.StringVar()
dr_entry_val[7] = tk.StringVar()

dr_entry_val[0].set("")
dr_entry_val[1].set("")
dr_entry_val[2].set("")
dr_entry_val[3].set("")
dr_entry_val[4].set("")
dr_entry_val[5].set("")
dr_entry_val[6].set("")
dr_entry_val[7].set("")

dr_entry1 = tk.Entry(dr_ax_lab_frame, textvariable=dr_entry_val[0], width=25)
dr_entry1.insert(tk.END,"")
dr_entry1.grid(column=0, row=0, pady = 0)

dr_entry2 = tk.Entry(dr_ax_lab_frame, textvariable=dr_entry_val[1], width=25)
dr_entry2.insert(tk.END,"")
dr_entry2.grid(column=0, row=1, pady = 4)

dr_entry3 = tk.Entry(dr_ax_lab_frame, textvariable=dr_entry_val[2], width=25)
dr_entry3.insert(tk.END,"")
dr_entry3.grid(column=0, row=2, pady = 4)

dr_entry4 = tk.Entry(dr_ax_lab_frame, textvariable=dr_entry_val[3], width=25)
dr_entry4.insert(tk.END,"")
dr_entry4.grid(column=0, row=3, pady = 4)

dr_entry5 = tk.Entry(dr_ax_lab_frame, textvariable=dr_entry_val[4], width=25)
dr_entry5.insert(tk.END,"")
dr_entry5.grid(column=0, row=4, pady = 4)

dr_entry6 = tk.Entry(dr_ax_lab_frame, textvariable=dr_entry_val[5], width=25)
dr_entry6.insert(tk.END,"")
dr_entry6.grid(column=0, row=5, pady = 4)

dr_entry7 = tk.Entry(dr_ax_lab_frame, textvariable=dr_entry_val[6], width=25)
dr_entry7.insert(tk.END,"")
dr_entry7.grid(column=0, row=6, pady = 4)

dr_entry8 = tk.Entry(dr_ax_lab_frame, textvariable=dr_entry_val[7], width=25)
dr_entry8.insert(tk.END,"")
dr_entry8.grid(column=0, row=7, pady = 4)

dr_entry_label1 = tk.Label( dr_ax_lab_frame, text = "軸1")
dr_entry_label1.grid(column=1, row=0)

dr_entry_label2 = tk.Label( dr_ax_lab_frame, text = "軸2")
dr_entry_label2.grid(column=1, row=1)

dr_entry_label3 = tk.Label( dr_ax_lab_frame, text = "軸3")
dr_entry_label3.grid(column=1, row=2)

dr_entry_label4 = tk.Label( dr_ax_lab_frame, text = "軸4")
dr_entry_label4.grid(column=1, row=3)

dr_entry_label5 = tk.Label( dr_ax_lab_frame, text = "軸5")
dr_entry_label5.grid(column=1, row=4)

dr_entry_label6 = tk.Label( dr_ax_lab_frame, text = "軸6")
dr_entry_label6.grid(column=1, row=5)

dr_entry_label7 = tk.Label( dr_ax_lab_frame, text = "軸7")
dr_entry_label7.grid(column=1, row=6)

dr_entry_label8 = tk.Label( dr_ax_lab_frame, text = "軸8")
dr_entry_label8.grid(column=1, row=7)

#ドライブ > 5.番号指定/状態/ポート出力値
dr_etc_frame = tk.Frame(dr_parameter, width=300, height=300 )
dr_etc_frame.grid(column=3, row=0, sticky='n' )

# 番号指定/状態/ポート出力値 サブフレーム
dr_etc_spin_frame = tk.LabelFrame(dr_etc_frame, width=300, height=300,text="番号指定")
dr_etc_spin_frame.grid(column=0, row=0, sticky='new', pady = 0)

dr_etc_radio_frame = tk.LabelFrame(dr_etc_frame, width=300, height=300,text="状態")
dr_etc_radio_frame.grid(column=0, row=1, sticky='new', pady = 5)

dr_etc_entry_frame = tk.LabelFrame(dr_etc_frame, width=300, height=300,text="ポート出力値")
dr_etc_entry_frame.grid(column=0, row=2, sticky='new', pady = 5)

dr_etc_label_str = tk.StringVar()
dr_etc_label_str.set("")

dr_etc_label_frame = tk.Label(dr_etc_frame, textvariable=dr_etc_label_str, justify='left')
dr_etc_label_frame.grid(column=0, row=3, sticky='new', pady = 5)

#ドライブ > 5.番号指定/状態/ポート出力値 > 番号指定
dr_etc_spin_var = StringVar()
dr_etc_spin_var.set(0)

dr_etc_spinbox = tk.Spinbox(dr_etc_spin_frame, textvariable=dr_etc_spin_var, from_=0, to=8, increment=1, width=15)
dr_etc_spinbox.grid(column=0, row=0, sticky='new' )

#ドライブ >5.番号指定/状態/ポート出力値 > 状態
# Radioボタンgroup設定
dr_radio_var = tk.IntVar()
dr_radio_var.set(0)

dr_radio_btn1 = tk.Radiobutton(dr_etc_radio_frame, text = u"OFF", variable = dr_radio_var, value = 0, command = None)
dr_radio_btn1.grid(column=0, row=0, sticky='n')

dr_radio_btn2 = tk.Radiobutton(dr_etc_radio_frame, text = u"ON ", variable = dr_radio_var, value = 1, command = None)
dr_radio_btn2.grid(column=0, row=1, sticky='n')

#ドライブ >5.番号指定/状態/ポート出力値 > ポート出力値
dr_etc_entry1 = tk.Entry(dr_etc_entry_frame, width=17)
dr_etc_entry1.insert(tk.END,"0xFFFF")
dr_etc_entry1.grid(column=0, row=0, sticky='new' )

#アクティブ パターンを初期化
dr_input_pattern_change(1)

# ドライブ > (状態)1.ボタン ：分類はstateだがdr_stateに配置する
st_button_str = tk.StringVar()
st_button_str.set("状態取得と移動待ち")

st_button = tk.Button(dr_state, textvariable=st_button_str, command=st_button_exec)
st_button.grid(column=1, row=2, sticky='nw', pady = 5 )

# 状態
#state = tk.LabelFrame(root, width=300, height=300,text="状態系")
#state.grid(column=0, row=1)

# 状態 > 2.状態
st_state = tk.Frame(state, width=300, height=300 )
st_state.grid(column=0, row=0, sticky='n' )

st_label_str = ["軸1","軸2","軸3","軸4","軸5","軸6","軸7","軸8"]
st_lavel = [0] * len(st_label_str)

st_busy_str = "BUSY"
st_busy_var = [False] * SSMC_AXES_NUM
st_busy = [0] * SSMC_AXES_NUM

st_ml_ccw_str = "ML-CCW"
st_ml_ccw_var = [False] * SSMC_AXES_NUM
st_ml_ccw = [0] * SSMC_AXES_NUM

st_sl_ccw_str = "SL-CCW"
st_sl_ccw_var = [False] * SSMC_AXES_NUM
st_sl_ccw = [0] * SSMC_AXES_NUM

st_ml_cw_str = "ML-CW"
st_ml_cw_var = [False] * SSMC_AXES_NUM
st_ml_cw = [0] * SSMC_AXES_NUM

st_sl_cw_str = "SL-CW"
st_sl_cw_var = [False] * SSMC_AXES_NUM
st_sl_cw = [0] * SSMC_AXES_NUM

st_comp_org_str = "COMP-ORG"
st_comp_org_var = [False] * SSMC_AXES_NUM
st_comp_org = [0] * SSMC_AXES_NUM

st_entry_val = [""] * SSMC_AXES_NUM
st_entry = [0] * SSMC_AXES_NUM

for i in range(len(st_label_str)):
    # 状態 > 2.状態 > 1.ラベル
    st_lavel[i] = tk.Label( st_state, text = st_label_str[i])
    st_lavel[i].grid(column=0, row=i, sticky='n' )

    # 状態 > 2.状態 > 2.BUSY
    st_busy_var[i] = tk.BooleanVar()
    st_busy[i] = tk.Checkbutton(st_state, variable=st_busy_var[i], text=st_busy_str) 
    st_busy[i].grid(column=1, row=i, sticky='n' )

    # 状態 > 2.状態 > 3.ML-CCW
    st_ml_ccw_var[i] = tk.BooleanVar()
    st_ml_ccw[i] = tk.Checkbutton(st_state, variable=st_ml_ccw_var[i], text=st_ml_ccw_str) 
    st_ml_ccw[i].grid(column=2, row=i, sticky='n' )

    # 状態 > 2.状態 > 4.SL-CCW
    st_sl_ccw_var[i] = tk.BooleanVar()
    st_sl_ccw[i] = tk.Checkbutton(st_state, variable=st_sl_ccw_var[i], text=st_sl_ccw_str) 
    st_sl_ccw[i].grid(column=3, row=i, sticky='n' )

    # 状態 > 2.状態 > 5.ML-CW
    st_ml_cw_var[i] = tk.BooleanVar()
    st_ml_cw[i] = tk.Checkbutton(st_state, variable=st_ml_cw_var[i], text=st_ml_cw_str) 
    st_ml_cw[i].grid(column=4, row=i, sticky='n' )

    # 状態 > 2.状態 > 6.SL-CW
    st_sl_cw_var[i] = tk.BooleanVar()
    st_sl_cw[i] = tk.Checkbutton(st_state, variable=st_sl_cw_var[i], text=st_sl_cw_str) 
    st_sl_cw[i].grid(column=5, row=i, sticky='n' )

    # 状態 > 2.状態 > 7.COMP-ORG
    st_comp_org_var[i] = tk.BooleanVar()
    st_comp_org[i] = tk.Checkbutton(st_state, variable=st_comp_org_var[i], text=st_comp_org_str) 
    st_comp_org[i].grid(column=6, row=i, sticky='n' )

    # 状態 > 2.状態 > 8.座標
    st_entry_val[i] = tk.StringVar()
    st_entry[i] = tk.Entry(st_state, textvariable=st_entry_val[i], width=15)
    st_entry[i].insert(tk.END,"")
    st_entry[i].grid(column=7, row=i, sticky='n' )

# 状態 > 3.非常停止 / 4.ポート
st_etc = tk.Frame(state, width=300, height=300 )
st_etc.grid(column=0, row=1, sticky='nw' )

# 状態 > 3.非常停止
st_emergency = tk.LabelFrame(st_etc, width=300, height=300,text="非常停止")
st_emergency.grid(column=0, row=0, sticky='nw' )

# 状態 > 3.非常停止 > 1～2.ID
st_em_id_str = ["ID1","ID2"]
st_em_id_var = [False] * 2
st_em_id = [0] * 2

for i in range(len(st_em_id_str)):
    st_em_id_var[i] = tk.BooleanVar()
    st_em_id[i] = tk.Checkbutton(st_emergency, variable=st_em_id_var[i], text=st_em_id_str[i]) 
    st_em_id[i].grid(column=i, row=0, sticky='n' )

# 状態 > 4.ポート
st_port = tk.LabelFrame(st_etc, width=300, height=300,text="入力ポート")
st_port.grid(column=1, row=0, sticky='nw' )

st_port_id0 = tk.Frame(st_port, width=300, height=300 )
st_port_id0.grid(column=0, row=0, sticky='nw' )

st_port_id1 = tk.Frame(st_port, width=300, height=300 )
st_port_id1.grid(column=1, row=0, sticky='nw' )

st_port_id0_label = tk.Label( st_port_id0, text = "ID0")
st_port_id0_label.grid(column=0, row=1)

st_port_id1_label = tk.Label( st_port_id1, text = "ID1")
st_port_id1_label.grid(column=0, row=1)

st_port_id_str = ["7","6","5","4","3","2","1","0"]
st_port_axis0_label = [0] * len(st_port_id_str)
st_port_axis1_label = [0] * len(st_port_id_str)

st_port_id0_cb_var = [False] * SSMC_AXES_NUM
st_port_id0_cb = [0] * SSMC_AXES_NUM

st_port_id1_cb_var = [False] * SSMC_AXES_NUM
st_port_id1_cb = [0] * SSMC_AXES_NUM

for i in range(len(st_port_id_str)):
    # 状態 > 4.ポート > ID0
    st_port_axis0_label[i] = tk.Label( st_port_id0, text = st_port_id_str[i])
    st_port_axis0_label[i].grid(column=i+1, row=0, sticky='new' )

    st_port_id0_cb_var[i] = tk.BooleanVar()
    st_port_id0_cb[i] = tk.Checkbutton(st_port_id0, variable = st_port_id0_cb_var[i]) 
    st_port_id0_cb[i].grid(column=i+1, row=1, sticky='n' )

    # 状態 > 4.ポート > ID1
    st_port_axis1_label[i] = tk.Label( st_port_id1, text=st_port_id_str[i])
    st_port_axis1_label[i].grid(column=i+1, row=0, sticky='new' )

    st_port_id1_cb_var[i] = tk.BooleanVar()
    st_port_id1_cb[i] = tk.Checkbutton(st_port_id1, variable=st_port_id1_cb_var[i]) 
    st_port_id1_cb[i].grid(column=i+1, row=1, sticky='n' )

# コントローラID設定 > サブフレーム
unit_id = tk.Frame(unit, width=300, height=300 )
unit_id.grid(column=0, row=0, sticky='news' )

unit_link = tk.Frame(unit, width=300, height=300 )
unit_link.grid(column=1, row=0, sticky='news' )

unit_state = tk.LabelFrame(unit, width=300, height=300,text="接続状況")
unit_state.grid(column=2, row=0, sticky='news' )

unit_action = tk.Frame(unit, width=300, height=300 )
unit_action.grid(column=3, row=0, sticky='news' )

# コントローラID設定 > 1.ID0設定
unit_id_select_label_str = ["ID0設定","ID1設定"]
unit_id_select_str = ["ID0","ID1"]
unit_id_select_label = [0] * len(unit_id_select_label_str)

unit_id_select = [0] * len(unit_id_select_label_str)
unit_id_select_num = [0] * len(unit_id_select_label_str)

for i in range(len(unit_id_select_label_str)):
    # コントローラID設定 > 1.ID0設定 > ラベル
    unit_id_select_label[i] = tk.Label( unit_id, text = unit_id_select_label_str[i], width=10)
    unit_id_select_label[i].grid(column=i, row=0, sticky='e' )

    # コントローラID設定 > 1.ID0設定 > ドロップダウンリスト
    unit_id_select_num[i] = tk.StringVar()
    unit_id_select_num[i].set('')
    unit_id_select[i] = tk.OptionMenu( unit_id, unit_id_select_num[i], u'ID0', u'ID1')

    unit_id_select[i].grid(column=i, row=1, sticky='news' )

# コントローラID設定 > 2.LinkState
unit_link_val = tk.BooleanVar()

unit_link_val.set(False)

dr_checkbutton1 = tk.Checkbutton(unit_link,text=u"ID-01連携",variable=unit_link_val)
dr_checkbutton1.grid(column=0, row=0, sticky='news', pady = 20)

# コントローラID設定 > 3.接続状況
unit_state_id0 = tk.Frame(unit_state, width=300, height=300 )
unit_state_id0.grid(column=0, row=0, sticky='nw' )

unit_state_id1 = tk.Frame(unit_state, width=300, height=300 )
unit_state_id1.grid(column=1, row=0, sticky='nw' )

unit_state_id0_label = tk.Label( unit_state_id0, text = "ID0")
unit_state_id0_label.grid(column=0, row=1)

unit_state_id1_label = tk.Label( unit_state_id1, text = "ID1")
unit_state_id1_label.grid(column=0, row=1)

unit_state_id_str = ["1","2","3","4"]
unit_state_axis0_label = [0] * len(unit_state_id_str)
unit_state_axis1_label = [0] * len(unit_state_id_str)

unit_state_id0_cb_var = [False] * len(unit_state_id_str)
unit_state_id0_cb = [0] * len(unit_state_id_str)

unit_state_id1_cb_var = [False] * len(unit_state_id_str)
unit_state_id1_cb = [0] * len(unit_state_id_str)

for i in range(len(unit_state_id_str)):
    # コントローラID設定 > 3.接続状況 > ID0
    unit_state_axis0_label[i] = tk.Label( unit_state_id0, text = unit_state_id_str[i])
    unit_state_axis0_label[i].grid(column=i+1, row=0, sticky='new' )

    unit_state_id0_cb_var[i] = tk.BooleanVar()
    unit_state_id0_cb[i] = tk.Checkbutton(unit_state_id0, variable = unit_state_id0_cb_var[i]) 
    unit_state_id0_cb[i].grid(column=i+1, row=1, sticky='n' )

    # コントローラID設定 > 3.接続状況 > ID1
    unit_state_axis1_label[i] = tk.Label( unit_state_id1, text=unit_state_id_str[i])
    unit_state_axis1_label[i].grid(column=i+1, row=0, sticky='new' )

    unit_state_id1_cb_var[i] = tk.BooleanVar()
    unit_state_id1_cb[i] = tk.Checkbutton(unit_state_id1, variable=unit_state_id1_cb_var[i]) 
    unit_state_id1_cb[i].grid(column=i+1, row=1, sticky='n' )

# コントローラID設定 > 4.動作 > 4-1.実行
unit_button = tk.Button(unit_action, text="実行", width=8, command=unit_button_exec)
unit_button.grid(column=0, row=0, sticky='n', padx = 10)

# コントローラID設定 > 4.動作 > 4-1.更新
unit_update = tk.Button(unit_action, text="更新", width=8, command=unit_update_exec)
unit_update.grid(column=0, row=1, sticky='s', pady = 10, padx = 10)

# 設定/取得
#param = tk.LabelFrame(param_logger, width=300, height=300,text="設定/取得")
#param.grid(column=0, row=0, sticky='news' )

# 設定/取得 > 1.リスト
pr_list = tk.Frame(param, width=300, height=300 )
pr_list.grid(column=0, row=0, sticky='news' )

pr_ctl = tk.Frame(param, width=300, height=300 )
pr_ctl.grid(column=1, row=0, sticky='sw' )

pr_ver = StringVar(value=( 
         "ステージパラメータ",
         "単位",
         "フルステップ時の1パルス移動量",
         "ドライバ分割数",
         "起動速度(原点復帰時)",
         "駆動速度(原点復帰時)",
         "加速時間(原点復帰時)",
         "S字レート(原点復帰時)",
         "エンコーダ設定",
         "エンコーダ分解能比率分子",
         "エンコーダ分解能比率分母",
         "ソフトリミット有効/無効",
         "CW側ソフトリミット",
         "CCW側ソフトリミット",
         "ホームポジション",
         "ステージ機種ID",
         "起動速度(速度テーブル)",
         "駆動速度(速度テーブル)",
         "加速時間(速度テーブル)",
         "S字レート(速度テーブル)",
         "ティーチングポイント",
         "フラッシュメモリ書き込み") )

pr_listbox = tk.Listbox(pr_list,
                     listvariable=pr_ver,
                     width=30, height=22, command=None)
                     
pr_listbox.bind('<<ListboxSelect>>', pr_listbox_selected)
pr_listbox.grid(column=0, row=0, sticky='n' )

# 設定/取得 > 2.ティーチングポイント番号
pr_tp_num_labelframe = tk.LabelFrame(pr_ctl, width=300, height=300,text="ティーチング\r\nポイント番号")
pr_tp_num_labelframe.grid(column=0, row=0, sticky='sw', padx = 6, pady = 6)

pr_tp_num_spin_var = StringVar()
pr_tp_num_spin_var.set(0)

pr_tp_num_spinbox = tk.Spinbox(pr_tp_num_labelframe, textvariable=pr_tp_num_spin_var, from_=0, to=63, increment=1, width=8)
pr_tp_num_spinbox.grid(column=0, row=0, sticky='new' )

# 設定/取得 > 3.実行
pr_button = tk.Button(pr_ctl, text="パラメータの\r\n読み出しと\r\n書き込み", command=pr_button_exec)
pr_button.grid(column=0, row=1, sticky='new', padx = 6, pady = 6)

#logger = tk.Frame(param_logger, width=300, height=300 )
#logger.grid(column=0, row=1, sticky='news' )

# スクロールバー
logger_sb = tk.Scrollbar(logger)
logger_sb.grid(column=1, row=0, sticky='news' )

# テキストボックス
logger_text = tk.Text(logger, width=40, height=27, bg='gray95', relief="groove")
logger_text.grid(column=0, row=0, sticky='news' )

logger_text["yscrollcommand"] = logger_sb.set
logger_sb["command"]=logger_text.yview

# タイマー起動
worker_thread = Worker(st_get_state)
worker_thread.start()

# SSMotionControl読み込み
ssmc = SSMotionControl.SSMotionControl();

if not ssmc.Open():
    messagebox.showinfo("", "Open() return False")
    on_closing()
    
root.mainloop()

